library angular2.src.core.compiler.view_pool;

import "package:angular2/di.dart" show Inject, Injectable, OpaqueToken;
import "package:angular2/src/facade/collection.dart"
    show ListWrapper, MapWrapper, Map, List;
import "package:angular2/src/facade/lang.dart" show isPresent, isBlank;
import "view.dart" as viewModule;

const APP_VIEW_POOL_CAPACITY =
    const OpaqueToken("AppViewPool.viewPoolCapacity");
@Injectable()
class AppViewPool {
  num _poolCapacityPerProtoView;
  Map<viewModule.AppProtoView, List<viewModule.AppView>> _pooledViewsPerProtoView;
  AppViewPool(@Inject(APP_VIEW_POOL_CAPACITY) poolCapacityPerProtoView) {
    this._poolCapacityPerProtoView = poolCapacityPerProtoView;
    this._pooledViewsPerProtoView = MapWrapper.create();
  }
  viewModule.AppView getView(viewModule.AppProtoView protoView) {
    var pooledViews = MapWrapper.get(this._pooledViewsPerProtoView, protoView);
    if (isPresent(pooledViews) && pooledViews.length > 0) {
      return ListWrapper.removeLast(pooledViews);
    }
    return null;
  }
  bool returnView(viewModule.AppView view) {
    var protoView = view.proto;
    var pooledViews = MapWrapper.get(this._pooledViewsPerProtoView, protoView);
    if (isBlank(pooledViews)) {
      pooledViews = [];
      MapWrapper.set(this._pooledViewsPerProtoView, protoView, pooledViews);
    }
    var haveRemainingCapacity =
        pooledViews.length < this._poolCapacityPerProtoView;
    if (haveRemainingCapacity) {
      ListWrapper.push(pooledViews, view);
    }
    return haveRemainingCapacity;
  }
}
