library angular2.src.forms.directives.checkbox_value_accessor;

import "package:angular2/angular2.dart" show Directive;
import "control_directive.dart" show ControlDirective;
import "control_value_accessor.dart" show ControlValueAccessor;

/**
 * The accessor for writing a value and listening to changes on a checkbox input element.
 *
 *
 *  # Example
 *  ```
 *  <input type="checkbox" [ng-control]="rememberLogin">
 *  ```
 *
 * @exportedAs angular2/forms
 */
@Directive(
    selector: "input[type=checkbox][ng-control],input[type=checkbox][ng-form-control],input[type=checkbox][ng-model]",
    hostListeners: const {
  "change": "onChange(\$event.target.checked)",
  "blur": "onTouched()"
},
    hostProperties: const {
  "checked": "checked",
  "cd.control?.untouched == true": "class.ng-untouched",
  "cd.control?.touched == true": "class.ng-touched",
  "cd.control?.pristine == true": "class.ng-pristine",
  "cd.control?.dirty == true": "class.ng-dirty",
  "cd.control?.valid == true": "class.ng-valid",
  "cd.control?.valid == false": "class.ng-invalid"
})
class CheckboxControlValueAccessor implements ControlValueAccessor {
  ControlDirective cd;
  bool checked;
  Function onChange;
  Function onTouched;
  CheckboxControlValueAccessor(this.cd) {
    this.onChange = (_) {};
    this.onTouched = (_) {};
    cd.valueAccessor = this;
  }
  writeValue(value) {
    this.checked = value;
  }
  void registerOnChange(fn) {
    this.onChange = fn;
  }
  void registerOnTouched(fn) {
    this.onTouched = fn;
  }
}
