library angular2.src.forms.directives.default_value_accessor;

import "package:angular2/angular2.dart" show Directive;
import "control_directive.dart" show ControlDirective;
import "control_value_accessor.dart" show ControlValueAccessor;

/**
 * The default accessor for writing a value and listening to changes that is used by a
 * {@link Control} directive.
 *
 * This is the default strategy that Angular uses when no other accessor is applied.
 *
 *  # Example
 *  ```
 *  <input type="text" [ng-form-control]="loginControl">
 *  ```
 *
 * @exportedAs angular2/forms
 */
@Directive(
    selector: "input:not([type=checkbox])[ng-control],textarea[ng-control],input:not([type=checkbox])[ng-form-control],textarea[ng-form-control],input:not([type=checkbox])[ng-model],textarea[ng-model]",
    hostListeners: const {
  "change": "onChange(\$event.target.value)",
  "input": "onChange(\$event.target.value)",
  "blur": "onTouched()"
},
    hostProperties: const {
  "value": "value",
  "cd.control?.untouched == true": "class.ng-untouched",
  "cd.control?.touched == true": "class.ng-touched",
  "cd.control?.pristine == true": "class.ng-pristine",
  "cd.control?.dirty == true": "class.ng-dirty",
  "cd.control?.valid == true": "class.ng-valid",
  "cd.control?.valid == false": "class.ng-invalid"
})
class DefaultValueAccessor implements ControlValueAccessor {
  ControlDirective cd;
  var value = null;
  Function onChange;
  Function onTouched;
  DefaultValueAccessor(this.cd) {
    this.onChange = (_) {};
    this.onTouched = (_) {};
    cd.valueAccessor = this;
  }
  writeValue(value) {
    this.value = value;
  }
  void registerOnChange(fn) {
    this.onChange = fn;
  }
  void registerOnTouched(fn) {
    this.onTouched = fn;
  }
}
