library angular2.src.forms.directives.form_control_directive;

import "package:angular2/src/facade/collection.dart" show StringMapWrapper;
import "package:angular2/src/facade/async.dart"
    show EventEmitter, ObservableWrapper;
import "package:angular2/angular2.dart" show Directive, Ancestor, onChange;
import "package:angular2/di.dart" show Binding;
import "control_directive.dart" show ControlDirective;
import "../model.dart" show Control;
import "shared.dart" show setUpControl;

const formControlBinding =
    const Binding(ControlDirective, toAlias: FormControlDirective);
/**
 * Binds a control to a DOM element.
 *
 * # Example
 *
 * In this example, we bind the control to an input element. When the value of the input element
 * changes, the value of
 * the control will reflect that change. Likewise, if the value of the control changes, the input
 * element reflects that
 * change.
 *
 * Here we use {@link formDirectives}, rather than importing each form directive individually, e.g.
 * `ControlDirective`, `ControlGroupDirective`. This is just a shorthand for the same end result.
 *
 *  ```
 * @Component({selector: "login-comp"})
 * @View({
 *      directives: [formDirectives],
 *      template: "<input type='text' [ng-form-control]='loginControl'>"
 *      })
 * class LoginComp {
 *  loginControl:Control;
 *
 *  constructor() {
 *    this.loginControl = new Control('');
 *  }
 * }
 *
 *  ```
 *
 * @exportedAs angular2/forms
 */
@Directive(
    selector: "[ng-form-control]",
    hostInjector: const [formControlBinding],
    properties: const ["form: ng-form-control", "model: ng-model"],
    events: const ["ngModel"],
    lifecycle: const [onChange])
class FormControlDirective extends ControlDirective {
  Control form;
  EventEmitter ngModel;
  bool _added;
  dynamic model;
  FormControlDirective() : super() {
    /* super call moved to initializer */;
    this.ngModel = new EventEmitter();
    this._added = false;
  }
  onChange(c) {
    if (!this._added) {
      setUpControl(this.form, this);
      this.form.updateValidity();
      this._added = true;
    }
    if (StringMapWrapper.contains(c, "model")) {
      this.form.updateValue(this.model);
    }
  }
  Control get control {
    return this.form;
  }
  List<String> get path {
    return [];
  }
  void viewToModelUpdate(dynamic newValue) {
    ObservableWrapper.callNext(this.ngModel, newValue);
  }
}
