library angular2.src.forms.directives.form_model_directive;

import "package:angular2/src/facade/collection.dart" show List, ListWrapper;
import "package:angular2/angular2.dart" show Directive, onChange;
import "package:angular2/di.dart" show Binding;
import "control_directive.dart" show ControlDirective;
import "control_group_directive.dart" show ControlGroupDirective;
import "control_container_directive.dart" show ControlContainerDirective;
import "form_directive.dart" show FormDirective;
import "../model.dart" show Control, ControlGroup;
import "shared.dart" show setUpControl;

const formDirectiveBinding =
    const Binding(ControlContainerDirective, toAlias: FormModelDirective);
/**
 * Binds a control group to a DOM element.
 *
 * # Example
 *
 * In this example, we bind the control group to the form element, and we bind the login and
 * password controls to the
 * login and password elements.
 *
 * Here we use {@link formDirectives}, rather than importing each form directive individually, e.g.
 * `ControlDirective`, `ControlGroupDirective`. This is just a shorthand for the same end result.
 *
 *  ```
 * @Component({selector: "login-comp"})
 * @View({
 *      directives: [formDirectives],
 *      template: "<form [ng-form-model]='loginForm'>" +
 *              "Login <input type='text' ng-control='login'>" +
 *              "Password <input type='password' ng-control='password'>" +
 *              "<button (click)="onLogin()">Login</button>" +
 *              "</form>"
 *      })
 * class LoginComp {
 *  loginForm:ControlGroup;
 *
 *  constructor() {
 *    this.loginForm = new ControlGroup({
 *      login: new Control(""),
 *      password: new Control("")
 *    });
 *  }
 *
 *  onLogin() {
 *    // this.loginForm.value
 *  }
 * }
 *
 *  ```
 *
 * @exportedAs angular2/forms
 */
@Directive(
    selector: "[ng-form-model]",
    hostInjector: const [formDirectiveBinding],
    properties: const ["form: ng-form-model"],
    lifecycle: const [onChange])
class FormModelDirective extends ControlContainerDirective
    implements FormDirective {
  ControlGroup form = null;
  List<ControlDirective> directives;
  FormModelDirective() : super() {
    /* super call moved to initializer */;
    this.directives = [];
  }
  onChange(_) {
    this._updateDomValue();
  }
  FormDirective get formDirective {
    return this;
  }
  List<String> get path {
    return [];
  }
  void addControl(ControlDirective dir) {
    dynamic c = this.form.find(dir.path);
    setUpControl(c, dir);
    c.updateValidity();
    ListWrapper.push(this.directives, dir);
  }
  Control getControl(ControlDirective dir) {
    return (this.form.find(dir.path) as Control);
  }
  void removeControl(ControlDirective dir) {
    ListWrapper.remove(this.directives, dir);
  }
  addControlGroup(ControlGroupDirective dir) {}
  removeControlGroup(ControlGroupDirective dir) {}
  void updateModel(ControlDirective dir, dynamic value) {
    var c = (this.form.find(dir.path) as Control);
    c.updateValue(value);
  }
  _updateDomValue() {
    ListWrapper.forEach(this.directives, (dir) {
      dynamic c = this.form.find(dir.path);
      dir.valueAccessor.writeValue(c.value);
    });
  }
}
