library angular2.src.forms.directives.template_driven_form_directive;

import "package:angular2/src/facade/async.dart" show PromiseWrapper;
import "package:angular2/src/facade/collection.dart"
    show StringMapWrapper, List, ListWrapper;
import "package:angular2/src/facade/lang.dart" show isPresent;
import "package:angular2/src/core/annotations/decorators.dart" show Directive;
import "package:angular2/di.dart" show Binding;
import "control_directive.dart" show ControlDirective;
import "form_directive.dart" show FormDirective;
import "control_group_directive.dart" show ControlGroupDirective;
import "control_container_directive.dart" show ControlContainerDirective;
import "../model.dart" show AbstractControl, ControlGroup, Control;
import "shared.dart" show setUpControl;

const formDirectiveBinding = const Binding(ControlContainerDirective,
    toAlias: TemplateDrivenFormDirective);
@Directive(
    selector: "form:not([ng-no-form]):not([ng-form-model]),ng-form,[ng-form]",
    hostInjector: const [formDirectiveBinding])
class TemplateDrivenFormDirective extends ControlContainerDirective
    implements FormDirective {
  ControlGroup form;
  TemplateDrivenFormDirective() : super() {
    /* super call moved to initializer */;
    this.form = new ControlGroup({});
  }
  FormDirective get formDirective {
    return this;
  }
  List<String> get path {
    return [];
  }
  Map<String, AbstractControl> get controls {
    return this.form.controls;
  }
  dynamic get value {
    return this.form.value;
  }
  void addControl(ControlDirective dir) {
    this._later((_) {
      var group = this._findContainer(dir.path);
      var c = new Control("");
      setUpControl(c, dir);
      group.addControl(dir.name, c);
    });
  }
  Control getControl(ControlDirective dir) {
    return (this.form.find(dir.path) as Control);
  }
  void removeControl(ControlDirective dir) {
    this._later((_) {
      var c = this._findContainer(dir.path);
      if (isPresent(c)) c.removeControl(dir.name);
    });
  }
  void addControlGroup(ControlGroupDirective dir) {
    this._later((_) {
      var group = this._findContainer(dir.path);
      var c = new ControlGroup({});
      group.addControl(dir.name, c);
    });
  }
  void removeControlGroup(ControlGroupDirective dir) {
    this._later((_) {
      var c = this._findContainer(dir.path);
      if (isPresent(c)) c.removeControl(dir.name);
    });
  }
  void updateModel(ControlDirective dir, dynamic value) {
    this._later((_) {
      var c = (this.form.find(dir.path) as Control);
      c.updateValue(value);
    });
  }
  ControlGroup _findContainer(List<String> path) {
    ListWrapper.removeLast(path);
    return (this.form.find(path) as ControlGroup);
  }
  _later(fn) {
    var c = PromiseWrapper.completer();
    PromiseWrapper.then(c.promise, fn, (_) {});
    c.resolve(null);
  }
}
