library angular2.src.test_lib.utils;

import "package:angular2/src/facade/collection.dart"
    show List, ListWrapper, MapWrapper;
import "package:angular2/src/dom/dom_adapter.dart" show DOM;
import "package:angular2/src/facade/lang.dart"
    show isPresent, isString, RegExpWrapper, StringWrapper, RegExp;
import "package:angular2/src/render/dom/view/view.dart"
    show resolveInternalDomView;

class Log {
  List<dynamic> _result;
  Log() {
    this._result = [];
  }
  void add(value) {
    ListWrapper.push(this._result, value);
  }
  fn(value) {
    return ([a1 = null, a2 = null, a3 = null, a4 = null, a5 = null]) {
      ListWrapper.push(this._result, value);
    };
  }
  String result() {
    return ListWrapper.join(this._result, "; ");
  }
}
List<dynamic> viewRootNodes(view) {
  return resolveInternalDomView(view.render).rootNodes;
}
queryView(view, String selector) {
  var rootNodes = viewRootNodes(view);
  for (var i = 0; i < rootNodes.length; ++i) {
    var res = DOM.querySelector(rootNodes[i], selector);
    if (isPresent(res)) {
      return res;
    }
  }
  return null;
}
dispatchEvent(element, eventType) {
  DOM.dispatchEvent(element, DOM.createEvent(eventType));
}
el(String html) {
  return DOM.firstChild(DOM.content(DOM.createTemplate(html)));
}
var _RE_SPECIAL_CHARS = [
  "-",
  "[",
  "]",
  "/",
  "{",
  "}",
  "\\",
  "(",
  ")",
  "*",
  "+",
  "?",
  ".",
  "^",
  "\$",
  "|"
];
var _ESCAPE_RE =
    RegExpWrapper.create('''[\\${ _RE_SPECIAL_CHARS . join ( "\\" )}]''');
RegExp containsRegexp(String input) {
  return RegExpWrapper.create(StringWrapper.replaceAllMapped(
      input, _ESCAPE_RE, (match) => '''\\${ match [ 0 ]}'''));
}
String normalizeCSS(String css) {
  css = StringWrapper.replaceAll(css, RegExpWrapper.create("\\s+"), " ");
  css = StringWrapper.replaceAll(css, RegExpWrapper.create(":\\s"), ":");
  css = StringWrapper.replaceAll(css, RegExpWrapper.create("\\'"), "\"");
  css = StringWrapper.replaceAllMapped(css,
      RegExpWrapper.create("url\\(\\\"(.+)\\\"\\)"),
      (match) => '''url(${ match [ 1 ]})''');
  css = StringWrapper.replaceAllMapped(css,
      RegExpWrapper.create("\\[(.+)=([^\"\\]]+)\\]"),
      (match) => '''[${ match [ 1 ]}="${ match [ 2 ]}"]''');
  return css;
}
var _singleTagWhitelist = ["br", "hr", "input"];
String stringifyElement(el) {
  var result = "";
  if (DOM.isElementNode(el)) {
    var tagName = StringWrapper.toLowerCase(DOM.tagName(el));
    // Opening tag
    result += '''<${ tagName}''';
    // Attributes in an ordered way
    var attributeMap = DOM.attributeMap(el);
    var keys = ListWrapper.create();
    MapWrapper.forEach(attributeMap, (v, k) {
      ListWrapper.push(keys, k);
    });
    ListWrapper.sort(keys);
    for (var i = 0; i < keys.length; i++) {
      var key = keys[i];
      var attValue = MapWrapper.get(attributeMap, key);
      if (!isString(attValue)) {
        result += ''' ${ key}''';
      } else {
        result += ''' ${ key}="${ attValue}"''';
      }
    }
    result += ">";
    // Children
    var children = DOM.childNodes(DOM.templateAwareRoot(el));
    for (var j = 0; j < children.length; j++) {
      result += stringifyElement(children[j]);
    }
    // Closing tag
    if (!ListWrapper.contains(_singleTagWhitelist, tagName)) {
      result += '''</${ tagName}>''';
    }
  } else {
    result += DOM.getText(el);
  }
  return result;
}
